import os
from json.decoder import JSONDecodeError
from pathlib import Path
from typing import Union
from jsonschema import Draft202012Validator


# class to perform validation
class Dataset_Validator:
    """
    Class to check JSON dataset and ensure the data is labeled for training.
    """

    def __init__(
        self,
        prompt_column: Union[str, None] = None,
        context_column: str = "context",
        label_column: str = "completion",
        custom_schema: Union[dict, None] = None,
        verbose: bool = False,
        pd_json_kwargs: Union[dict, None] = None,
    ) -> None:
        """Init for dataset validator class

        Args:
            prompt_column (str, optional): Dataset prompt column name. Defaults to None.
            context_column (str, optional): Dataset context column name. Defaults to "context".
            label_column (str, optional): Dataset label column name. Defaults to "completion".
            custom_schema (dict, optional): Your very own custom jsonschema to use instead. Defaults to None.
            verbose (bool, optional): Verbose or quiet. Defaults to False.
            pd_json_kwargs, (dict, optional): kwargs for pandas when loading dataset from JSON. Defaults to None.
        """
        self.verbose = verbose

        if pd_json_kwargs is None:
            pd_json_kwargs = {"lines": False, "orient": None}
        self.pd_json_kwargs = pd_json_kwargs

        # Init data validation object
        if not custom_schema:
            self.schema = {
                # "type": "object",
                "properties": {
                    prompt_column: {"type": "string"},
                    context_column: {"type": ["string", "null"]},
                    label_column: {"type": "string"},
                    "source": {"type": ["string", "null"]},
                },
                "required": ["prompt"],
                "additionalProperties": False,
            }
        else:
            self.schema = custom_schema

        Draft202012Validator.check_schema(self.schema)
        self.validator = Draft202012Validator(self.schema)

    def __call__(
        self, filename: Path, return_dataframe: bool = False
    ):
        # Check if the file exists
        assert os.path.exists(filename), f"The file '{filename}' could not be found"
        if self.verbose:
            print("Found dataset file.")

        # Check the json is valid for training.
        if return_dataframe:
            df_data = self.load_json_file(filename=filename)
            return df_data if self.is_valid_for_training(df=df_data) else True
        else:
            return False

    def is_valid_for_training(self, df) -> bool:
        if self.verbose:
            print(f"Found {len(df)} data entries")
        for _, row in df.iterrows():
            row = row.dropna()
            pt = row.to_json()
            self.validator.validate(pt)

        if self.verbose:
            print("All entries in the dataset passed validation ✅")
        return True

    def load_json_file(self, filename):
        try:
            import pandas as pd
        except ImportError as error:
            print("Pandas is required for loading JSON files. Please install 'pandas' with pip or conda.")
            raise error
        
        extension = os.path.splitext(filename)[1]

        if extension in {".json", ".jsonl"}:
            if extension == ".jsonl":
                self.pd_json_kwargs["orient"] = "records"
            if self.verbose:
                print(f"Reading in JSON file at {filename}")
            df = pd.read_json(filename, **self.pd_json_kwargs)
        else:
            try:
                df = pd.read_json(filename, **self.pd_json_kwargs)
            except JSONDecodeError as error:
                print("Cannot decode your file. Ensure it is valid json or jsonl")
                raise error

        return df
